/**
    @file

    @brief
	(Brief Text)

	(Description Text)
**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2015, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/
#include "shell.h"

#include <ctype.h>

/* CONSTANTS ***********************************************************************/

#define BUFFER_SIZE (64)

#define ASCII_BS (8)
#define ASCII_DELETE (127)

static const char* STR_PROMPT = ">";

/* GLOBAL VARIABLES ****************************************************************/

/* LOCAL VARIABLES *****************************************************************/

static char buffer[BUFFER_SIZE];
static uint8_t bufferptr;
static interpreter_cbptr_t cmd_callback;
static uint8_t echo;

/* MACROS **************************************************************************/

#define GO_UNTIL(var,start,cond,size) for(var=start; (var<size) && (cond); var++)
#define BUF_GO_UNTIL(var,start,func,size) GO_UNTIL(var,start,func((int)buffer[var]),size)

/* LOCAL FUNCTIONS / INLINES *******************************************************/

void shell_process(char* str, int size);

/* FUNCTIONS ***********************************************************************/

void shell_init(interpreter_cbptr_t callback)
{
    /* Enable the UART Device... */
    sys_enable(sys_device_uart0);
    /* Make GPIO48 function as UART0_TXD and GPIO49 function as UART0_RXD... */
    gpio_function(48, pad_uart0_txd); /* UART0 TXD */
    gpio_function(49, pad_uart0_rxd); /* UART0 RXD */
    uart_open(UART0,                    /* Device */
              1,                        /* Prescaler = 1 */
              UART_DIVIDER_19200_BAUD,  /* Divider = 1302 */
              uart_data_bits_8,         /* No. Data Bits */
              uart_parity_none,         /* Parity */
              uart_stop_bits_1);        /* No. Stop Bits */

    /* Print out a welcome message... */
        uart_puts(UART0,
            "\x1B[2J" /* ANSI/VT100 - Clear the Screen */
            "\x1B[H"  /* ANSI/VT100 - Move Cursor to Home */
            "(C) Copyright 2015, Future Technology Devices International Ltd. \r\n"
            "--------------------------------------------------------------------- \r\n"
            "Welcome to UART to GPIO Bridge Application... \r\n"
            "\r\n"
            "Allows the GPIO pins on the FT90x chip to be controlled via commands \r\n"
        	"issued over a UART.\r\n"
        	"Type ?<CR> to display the help text which lists the commands. \r\n"
            "--------------------------------------------------------------------- \r\n"
            );

    cmd_callback = callback;

    shell_echo(1);

    //shell_puts(STR_PROMPT);
}

void shell_echo(uint8_t state)
{
    if (state)
    {
        echo = 1;
    }
    else
    {
        echo = 0;
    }
}

void shell_write(uint8_t c)
{
    if (echo)
        uart_write(UART0, c);
}

void shell_writef(uint8_t c)
{
    uart_write(UART0, c);
}

void shell_puts(const char* str)
{
    if (echo)
        uart_puts(UART0, str);
}

void shell_putsf(const char* str)
{
    uart_puts(UART0, str);
}

void shell_tick(void)
{
    uint8_t c;
    static uint8_t vt100 = 0, vt100e = 0;
    static char vt100_str[16] = {0};

    /* Read in a character */
    uart_read(UART0, &c);

    /* Catch VT100 terminal codes (Arrow keys, ESC codes, etc) */
    if (!vt100 && c==0x1B)
    {
        vt100 = 1;
        vt100e = 0;
    }
    else if (vt100)
    {
        vt100++;

        if (vt100 == 2 && c=='[')
        {
            vt100e = 1;
        }

        vt100_str[vt100-2] = c;

        if ((!vt100e) || (vt100e && isalpha((int)c)))
        {
            vt100_str[vt100-1] = 0;
            vt100 = 0;
        }
    }
    else
    {

        /* A normal character arrived, interpret */
        switch(c)
        {
            case '\r': /* Enter Key */
                {
                    /* Terminate the string */
                    buffer[bufferptr++] = 0;

                    shell_puts("\r\n");
                    shell_process(buffer, bufferptr);
                    shell_puts(STR_PROMPT);

                    bufferptr = 0;
                }
                break;

            case ASCII_BS: /* Backspace */
            case ASCII_DELETE: /* Backspace */
                {
                    if (bufferptr > 0)
                    {
                        bufferptr--;
                        shell_write(c);
                    }
                }
                break;

            default:
                {
                    if ((isprint(c)) && (bufferptr < BUFFER_SIZE-1))
                    {
                        buffer[bufferptr++] = c;
                        shell_write(c);
                    }
                }
                break;
        }
    }
}

int shell_atoi(int *retint, char* str, int len, int radix)
{
    int val = 0, retval = 0;
    int i,j;
    static const char CHARLUT[] =
        {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

    if (str && (radix > 0) && (radix <= sizeof(CHARLUT)) && (len > 0))
    {
        for (i=0; (i<len) && (str[i]); ++i)
        {
            for (j=0; j<radix; ++j)
            {
                if (str[i] == CHARLUT[j])
                {
                    val = val * radix;
                    val += j;
                    break;
                }
            }

            if (j == radix)
            {
                /* Unknown character - ERROR */
                i = len; /* Exit loop */
                retval = -1;
            }
        }
    }
    else
    {
        /* Either:
         * No valid string
         * Radix not between 1 and 16
         * or no positive length provided
         */
        retval = -1;
    }

    if (retval == 0)
    {
        *retint = val;
    }

    return retval;
}

void shell_process(char* str, int size)
{
    /* Return key pressed, interpret the command */
    uint8_t cmd_ptr, pin_ptr, pin_ptr2;
    char cmd = 0;
    int pin = -1;

    /* Ignore preceeding whitespace */
    GO_UNTIL(cmd_ptr, 0, isblank((int)str[cmd_ptr]), size);

    cmd = buffer[cmd_ptr];

    if (isprint((int)cmd))
    {
        /* We found a command letter */

        /* ignore any whitespace in between */
        GO_UNTIL(pin_ptr, cmd_ptr+1, isblank((int)str[pin_ptr]), size);

        /* Find the end of the number */
        GO_UNTIL(pin_ptr2, pin_ptr, isdigit((int)str[pin_ptr2]), size);

        if ((pin_ptr != pin_ptr2) && (pin_ptr != size) && (pin_ptr2 != size))
        {
            /* Decode the number */
           shell_atoi(&pin, buffer+pin_ptr, pin_ptr2-pin_ptr, 10);
        }

        if (cmd_callback)
            cmd_callback(cmd,pin);
    }
}
