package com.ftdi.uart2gpio.ui;

import java.util.concurrent.CountDownLatch;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JButton;

import com.ftdi.io.*;
import com.ftdi.swing.IconStore;
import com.ftdi.swing.JHyperlinkLabel;
import com.ftdi.util.AppProperties;


/**
 * Serial Port Chooser UI 
 */
public class SerialPortChooserUI {

	private JFrame frame;
	private String chosenPort;
	
	private final Icon iconWindow = IconStore.LoadIcon("/gfx/ftdiico.png");
	
	private static final String COPYRIGHT = AppProperties.getProperties().getProperty("ftdi.copyright");
	private static final String PRODUCT_PAGE = AppProperties.getProperties().getProperty("ftdi.product.url");
	private static final String APP_NOTE_PAGE = AppProperties.getProperties().getProperty("ftdi.appnote.url");
			
	
	private final CountDownLatch latchFinished = new CountDownLatch(1);

	/**
	 * Create the application.
	 * @param title The title of the window
	 */
	public SerialPortChooserUI(String title) {
		chosenPort = null;
		
		initialize(title);
	}

	/**
	 * Make the window visible and wait until a port
	 * has been chosen
	 * @return The port chosen or null
	 */
	public String getChosenPort()
	{
		frame.setVisible(true);
		
		try {
			latchFinished.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		return chosenPort;
	}

	/**
	 * Initialize the contents of the frame.
	 * @param title The title of the window
	 */
	private void initialize(String title) {
		frame = new JFrame();
		frame.setResizable(false);
		frame.setTitle(title);
		frame.setBounds(100, 100, 286, 180);
		frame.setLocationRelativeTo(null);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.getContentPane().setLayout(null);
		frame.setIconImage(((ImageIcon)iconWindow).getImage());
		// Add a window listener so that the main thread doesn't hang
		frame.addWindowListener(new WindowListener() {
			@Override
			public void windowOpened(WindowEvent e) {}
			@Override
			public void windowIconified(WindowEvent e) {}
			@Override
			public void windowDeiconified(WindowEvent e) {}
			@Override
			public void windowDeactivated(WindowEvent e) {}
			@Override
			public void windowClosing(WindowEvent e) {
				chosenPort = null;
				latchFinished.countDown();
			}
			@Override
			public void windowClosed(WindowEvent e) {}
			@Override
			public void windowActivated(WindowEvent e) {}
		});
		
		JComboBox<String> comboSerialPorts = new JComboBox<String>();
		comboSerialPorts.setBounds(10, 27, 161, 20);
		frame.getContentPane().add(comboSerialPorts);
		
		JLabel lblPleaseChooseA = new JLabel("Please choose a Serial Port:");
		lblPleaseChooseA.setBounds(10, 11, 273, 14);
		frame.getContentPane().add(lblPleaseChooseA);
		
		JButton btnOk = new JButton("OK");
		btnOkActionListener btnOkAL = new btnOkActionListener(comboSerialPorts); 
		btnOk.addActionListener(btnOkAL);
		btnOk.setBounds(10, 58, 89, 23);
		frame.getContentPane().add(btnOk);
		
		JButton btnRefresh = new JButton("Refresh");
		btnRefreshActionListener btnRefreshAL = new btnRefreshActionListener(comboSerialPorts); 
		btnRefresh.addActionListener(btnRefreshAL);
		btnRefreshAL.actionPerformed(null);
		btnRefresh.setBounds(181, 26, 89, 23);
		frame.getContentPane().add(btnRefresh);
		
		JLabel lblCopyright = new JLabel(COPYRIGHT);
		lblCopyright.setEnabled(false);
		lblCopyright.setBounds(10, 127, 260, 14);
		frame.getContentPane().add(lblCopyright);
		
		if (APP_NOTE_PAGE != null)
		{
			JLabel lblLinkAppNote = new JHyperlinkLabel(APP_NOTE_PAGE, "Application Note");
			lblLinkAppNote.setBounds(10, 92, 260, 14);
			frame.getContentPane().add(lblLinkAppNote);
		}
		
		if (PRODUCT_PAGE != null)
		{
			JLabel lblProductPage = new JHyperlinkLabel(PRODUCT_PAGE, "Product Page");
			lblProductPage.setBounds(11, 109, 260, 14);
			frame.getContentPane().add(lblProductPage);
		}

	}
	
	/**
	 * btnOk Action Listener
	 */
	class btnOkActionListener implements ActionListener {
		private JComboBox<String> ports;
		
		public btnOkActionListener(JComboBox<String> ports) {
			this.ports = ports;
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			chosenPort = (String) ports.getSelectedItem();
			
			if (chosenPort != null)
			{				
				frame.setVisible(false);
				latchFinished.countDown();
				frame.dispose();
			}
		}
	}
	
	/**
	 * btnRefresh Action Listener
	 */
	class btnRefreshActionListener implements ActionListener {
		private JComboBox<String> ports;
		
		public btnRefreshActionListener(JComboBox<String> ports) {
			this.ports = ports;
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			// Start up a new thread to get the serial ports
			// so the UI doesn't freeze.
			new Thread(new Runnable() {
				
				@Override
				public void run() {
					String[] listPorts = null;
					
					ports.setEnabled(false);
					
					String previouslySelected = (String) ports.getSelectedItem();
										
					ports.removeAllItems();
					
					listPorts = JSSCSerialPort.listPorts();
					
					System.out.print("Available ports found: ");
					
					int i;
					for(i = 0; i < listPorts.length; ++i)
					{
						ports.addItem(listPorts[i]);
						
						if ((listPorts[i]).equals(previouslySelected))
						{
							ports.setSelectedIndex(i);		
							System.out.print("*");
						}
						
						System.out.print("\""+listPorts[i]+"\" ");
					}
					System.out.println();
					
					ports.setEnabled(true);
				}
			}).start();
			
		}
	}
}
